import { GatewayIntentBits, Partials } from 'discord.js';
import { IClient, IClient$SetPresence } from './Client.types';
export default class Client {
    private clientId;
    private clientToken;
    private commandsPath;
    private eventsPath;
    private intents;
    private partials;
    private presences;
    private commands;
    private events;
    /**
     * Crear nueva instancia de un cliente Discord
     * @param {IClient} options - Opciones para crear nueva instancia de un nuevo cliente
     */
    constructor(options?: IClient);
    /**
     * Establecer ID del cliente Discord
     * @param clientId ID del cliente Discord
     */
    setClientId(clientId: string): this;
    /**
     * Establecer Token del cliente Discord
     * @param token Token del cliente Discord
     */
    setToken(token: string): this;
    /**
     * Establecer rutas para comandos
     * @param path Ruta de los comandos
     */
    setCommandsPath(path: string): this;
    /**
     * Establecer rutas para eventos
     * @param path Ruta de los eventos
     */
    setEventsPath(path: string): this;
    /**
     * Establecer intents del cliente Discord.
     * @param intents Arreglo de intents del cliente Discord
     */
    setIntents(intents: GatewayIntentBits[]): this;
    /**
     * Establecer partials del cliente Discord
     * @param partials - Arreglo de partials del cliente Discord
     * @returns
     */
    setPartials(partials: Partials[]): this;
    /**
     * Establecer presence dinámicos para el cliente Discord
     * @param options Opciones para crear presence dinámicos para el cliente Discord
     */
    setPresence(options: IClient$SetPresence): this;
    /**
     * Iniciar ejecución posterior a la configuración del cliente
     */
    start(): Promise<void>;
    private _loadCommands;
    private _loadEvents;
    private areIntentsValid;
    private arePartialsValid;
    private arePresenceValid;
}
