import type { TCommand, TCommand$Options } from "./Commands.types";
import { SlashCommandBuilder, SlashCommandSubcommandBuilder, SlashCommandSubcommandGroupBuilder } from "discord.js";
export default class Command {
    private _name;
    private _description;
    private _options;
    private _subcommands;
    private _subcommandsGroup;
    private _isNSFW;
    private _execute;
    get name(): string;
    get description(): string;
    get options(): TCommand$Options[];
    get subcommands(): SlashCommandSubcommandBuilder[];
    get subcommandsGroup(): SlashCommandSubcommandGroupBuilder[];
    get isNSFW(): boolean;
    get execute(): TCommand['execute'];
    /**
     * Crear nueva instancia de un nuevo comando para el cliente Discord
     * @param options Opciones para crear un nuevo bot
     */
    constructor(options?: TCommand);
    /**
     * Establecer nombre del comando
     * @param commandName Nombre del comando
     */
    setName(commandName: string): this;
    /**
     * Establecer descripción del comando
     * @param commandDescription Descripción del comando
     */
    setDescription(commandDescription: string): this;
    /**
     * Establecer opciones para el comando
     * @param options Opciones del comando
     */
    setOptions(options: TCommand$Options | TCommand$Options[]): Omit<this, 'setSubcommands' | 'setSubcommandsGroup'>;
    /**
     * Establecer subcomandos al comando
     * @param subcommands Arreglo de Subcomandos
     */
    setSubcommands(subcommands: SlashCommandSubcommandBuilder | SlashCommandSubcommandBuilder[]): Omit<this, 'setOptions'>;
    /**
     * Establecer grupo de subcomandos
     * @param subcommandsGroup Arreglo de grupo de subcommandos
     */
    setSubcommandsGroup(subcommandsGroup: SlashCommandSubcommandGroupBuilder[] | SlashCommandSubcommandGroupBuilder): Omit<this, "setOptions">;
    /**
     * Método para establecer la función a ejecutar cuando el comando sea invocado
     * @param executable Función a ejecutar
     */
    setExecutable(executable: TCommand['execute']): this;
    /**
     * Establecer si el comando es NSFW
     * @param isNSFW Estado NSFW del comando
     */
    setIsNSFW(isNSFW: boolean): this;
    private validateCommandOptions;
}
export declare const createSlashCommand: (command: Command) => SlashCommandBuilder;
