"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const discord_js_1 = require("discord.js");
const Debug_1 = require("../Debug");
const utils_1 = require("../utils");
class Event {
    //@ts-ignore
    _name = discord_js_1.Events.ClientReady;
    _execute = () => null;
    get name() {
        return this._name;
    }
    get execute() {
        return this._execute;
    }
    /**
     * Crear nueva instancia de un evento para el cliente Discord
     * @param options Opciones del evento
     */
    constructor(options) {
        if (typeof options === 'object') {
            if ('name' in options && typeof options.name === 'string' && !(0, utils_1.stringPropertyAreValid)(options.name))
                (0, Debug_1.debugError)(Debug_1.ErrorMessages.Event.Name);
            if ('execute' in options && typeof options.execute !== 'function')
                (0, Debug_1.debugError)(Debug_1.ErrorMessages.Event.Executable(this.name));
            this._name = options.name;
            this._execute = options.execute;
        }
    }
    /**
     * Establecer nombre del evento
     * @param name Nombre del evento a ejecutar
     */
    setName(name) {
        if (typeof name !== 'string')
            (0, Debug_1.debugError)(Debug_1.ErrorMessages.Event.Name);
        this._name = name;
        return {
            /**
             * Establecer función a ejecutar cuando el evento sea invocado
             * @param execute Función a ejecutar
             */
            setExecute: (execute) => {
                if (typeof execute !== 'function')
                    (0, Debug_1.debugError)(Debug_1.ErrorMessages.Event.Executable(this.name));
                this._execute = execute;
            }
        };
    }
}
exports.default = Event;
