"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.validateCommandOptions = exports.isInstanceOfTCommandOptions = void 0;
const index_1 = require("./index");
const Debug_1 = require("../Debug");
const discord_js_1 = require("discord.js");
const isInstanceOfTCommandOptions = (obj) => {
    return obj instanceof discord_js_1.SlashCommandAttachmentOption ||
        obj instanceof discord_js_1.SlashCommandStringOption ||
        obj instanceof discord_js_1.SlashCommandBooleanOption ||
        obj instanceof discord_js_1.SlashCommandChannelOption ||
        obj instanceof discord_js_1.SlashCommandIntegerOption ||
        obj instanceof discord_js_1.SlashCommandMentionableOption ||
        obj instanceof discord_js_1.SlashCommandNumberOption ||
        obj instanceof discord_js_1.SlashCommandRoleOption;
};
exports.isInstanceOfTCommandOptions = isInstanceOfTCommandOptions;
const validateCommandOptions = (options) => {
    if ('name' in options && (!(0, index_1.stringPropertyAreValid)(options.name) || options.name.includes(' ') || /(A-Z)/gi.test(options.name)))
        (0, Debug_1.debugError)(Debug_1.ErrorMessages.Command.Name);
    if ('description' in options && !(0, index_1.stringPropertyAreValid)(options.description))
        (0, Debug_1.debugError)(Debug_1.ErrorMessages.Command.Description(options.name));
    if ('options' in options && options.options && (!Array.isArray(options.options) || (options.options.length > 0 && !options.options.some(o => (0, exports.isInstanceOfTCommandOptions)(o)))))
        (0, Debug_1.debugError)(Debug_1.ErrorMessages.Command.Options(options.name));
    if ('subcommands' in options && options.subcommands && (!Array.isArray(options.subcommands) || (options.subcommands.length > 0 && !options.subcommands.some(s => s instanceof discord_js_1.SlashCommandSubcommandBuilder))))
        (0, Debug_1.debugError)(Debug_1.ErrorMessages.Command.Subcommand(options.name));
    if ('subcommandsGroup' in options && options.subcommandsGroup && (!Array.isArray(options.subcommandsGroup) || (options.subcommandsGroup.length > 0 && !options.subcommandsGroup.some(s => s instanceof discord_js_1.SlashCommandSubcommandGroupBuilder))))
        (0, Debug_1.debugError)(Debug_1.ErrorMessages.Command.SubcommandGroup(options.name));
    if ('isNSFW' in options && typeof options.isNSFW !== 'boolean')
        (0, Debug_1.debugError)(Debug_1.ErrorMessages.Command.NSFW(options.name));
    if ('executable' in options && typeof options.executable !== 'function')
        (0, Debug_1.debugError)(Debug_1.ErrorMessages.Command.Executable(options.name));
};
exports.validateCommandOptions = validateCommandOptions;
